@@ -1,14 +1,13 @@
 #include "common/printing.h"
 #include "common/jsonconfig.h"
+#include "common/time.h"
 #include "detection/uptime/uptime.h"
 #include "modules/uptime/uptime.h"
 #include "util/stringUtils.h"
 
-#define FF_UPTIME_NUM_FORMAT_ARGS 7
-
 void ffPrintUptime(FFUptimeOptions* options)
 {
-    FFUptimeResult result;
+    FFUptimeResult result = {};
 
     const char* error = ffDetectUptime(&result);
 
@@ -20,71 +19,38 @@ void ffPrintUptime(FFUptimeOptions* options)
 
     uint64_t uptime = result.uptime;
 
-    uint32_t milliseconds = (uint32_t) (uptime % 1000);
-    uptime /= 1000;
-    uint32_t seconds = (uint32_t) (uptime % 60);
-    uptime /= 60;
-    uint32_t minutes = (uint32_t) (uptime % 60);
-    uptime /= 60;
-    uint32_t hours = (uint32_t) (uptime % 24);
-    uptime /= 24;
-    uint32_t days = (uint32_t) uptime;
-
     if(options->moduleArgs.outputFormat.length == 0)
     {
         ffPrintLogoAndKey(FF_UPTIME_MODULE_NAME, 0, &options->moduleArgs, FF_PRINT_TYPE_DEFAULT);
+        FF_STRBUF_AUTO_DESTROY buffer = ffStrbufCreate();
+        ffParseDuration((uptime + 500) / 1000, &buffer);
 
-        if(days == 0 && hours == 0 && minutes == 0)
-        {
-            printf("%u seconds\n", seconds);
-            return;
-        }
-
-        if(days > 0)
-        {
-            printf("%u day", days);
-
-            if(days > 1)
-                putchar('s');
-
-            if(days >= 100)
-                fputs("(!)", stdout);
-
-            if(hours > 0 || minutes > 0)
-                fputs(", ", stdout);
-        }
-
-        if(hours > 0)
-        {
-            printf("%u hour", hours);
-
-            if(hours > 1)
-                putchar('s');
-
-            if(minutes > 0)
-                fputs(", ", stdout);
-        }
-
-        if(minutes > 0)
-        {
-            printf("%u min", minutes);
-
-            if(minutes > 1)
-                putchar('s');
-        }
-
-        putchar('\n');
+        ffStrbufPutTo(&buffer, stdout);
     }
     else
     {
-        FF_PRINT_FORMAT_CHECKED(FF_UPTIME_MODULE_NAME, 0, &options->moduleArgs, FF_PRINT_TYPE_DEFAULT, FF_UPTIME_NUM_FORMAT_ARGS, ((FFformatarg[]){
-            {FF_FORMAT_ARG_TYPE_UINT, &days},
-            {FF_FORMAT_ARG_TYPE_UINT, &hours},
-            {FF_FORMAT_ARG_TYPE_UINT, &minutes},
-            {FF_FORMAT_ARG_TYPE_UINT, &seconds},
-            {FF_FORMAT_ARG_TYPE_UINT, &milliseconds},
-            {FF_FORMAT_ARG_TYPE_UINT64, &result.uptime},
-            {FF_FORMAT_ARG_TYPE_UINT64, &result.bootTime},
+        uint32_t milliseconds = (uint32_t) (uptime % 1000);
+        uptime /= 1000;
+        uint32_t seconds = (uint32_t) (uptime % 60);
+        uptime /= 60;
+        uint32_t minutes = (uint32_t) (uptime % 60);
+        uptime /= 60;
+        uint32_t hours = (uint32_t) (uptime % 24);
+        uptime /= 24;
+        uint32_t days = (uint32_t) uptime;
+
+        FFTimeGetAgeResult age = ffTimeGetAge(result.bootTime, ffTimeGetNow());
+
+        FF_PRINT_FORMAT_CHECKED(FF_UPTIME_MODULE_NAME, 0, &options->moduleArgs, FF_PRINT_TYPE_DEFAULT, ((FFformatarg[]){
+            FF_FORMAT_ARG(days, "days"),
+            FF_FORMAT_ARG(hours, "hours"),
+            FF_FORMAT_ARG(minutes, "minutes"),
+            FF_FORMAT_ARG(seconds, "seconds"),
+            FF_FORMAT_ARG(milliseconds, "milliseconds"),
+            {FF_FORMAT_ARG_TYPE_STRING, ffTimeToShortStr(result.bootTime), "boot-time"},
+            FF_FORMAT_ARG(age.years, "years"),
+            FF_FORMAT_ARG(age.daysOfYear, "days-of-year"),
+            FF_FORMAT_ARG(age.yearsFraction, "years-fraction"),
         }));
     }
 }
@@ -137,36 +103,34 @@ void ffGenerateUptimeJsonResult(FF_MAYBE_UNUSED FFUptimeOptions* options, yyjson
 
     yyjson_mut_val* obj = yyjson_mut_obj_add_obj(doc, module, "result");
     yyjson_mut_obj_add_uint(doc, obj, "uptime", result.uptime);
-    yyjson_mut_obj_add_uint(doc, obj, "bootTime", result.bootTime);
+    yyjson_mut_obj_add_strcpy(doc, obj, "bootTime", ffTimeToFullStr(result.bootTime));
 }
 
-void ffPrintUptimeHelpFormat(void)
-{
-    FF_PRINT_MODULE_FORMAT_HELP_CHECKED(FF_UPTIME_MODULE_NAME, "{1} days {2} hours {3} mins", FF_UPTIME_NUM_FORMAT_ARGS, ((const char* []) {
-        "Days",
-        "Hours",
-        "Minutes",
-        "Seconds",
-        "Milliseconds",
-        "Uptime in unix timestamp (ms)",
-        "Boot time in unix timestamp (ms)",
-    }));
-}
+static FFModuleBaseInfo ffModuleInfo = {
+    .name = FF_UPTIME_MODULE_NAME,
+    .description = "Print how long system has been running",
+    .parseCommandOptions = (void*) ffParseUptimeCommandOptions,
+    .parseJsonObject = (void*) ffParseUptimeJsonObject,
+    .printModule = (void*) ffPrintUptime,
+    .generateJsonResult = (void*) ffGenerateUptimeJsonResult,
+    .generateJsonConfig = (void*) ffGenerateUptimeJsonConfig,
+    .formatArgs = FF_FORMAT_ARG_LIST(((FFModuleFormatArg[]) {
+        {"Days after boot", "days"},
+        {"Hours after boot", "hours"},
+        {"Minutes after boot", "minutes"},
+        {"Seconds after boot", "seconds"},
+        {"Milliseconds after boot", "milliseconds"},
+        {"Boot time in local timezone", "boot-time"},
+        {"Years integer after boot", "years"},
+        {"Days of year after boot", "days-of-year"},
+        {"Years fraction after boot", "years-fraction"},
+    }))
+};
 
 void ffInitUptimeOptions(FFUptimeOptions* options)
 {
-    ffOptionInitModuleBaseInfo(
-        &options->moduleInfo,
-        FF_UPTIME_MODULE_NAME,
-        "Print how long system has been running",
-        ffParseUptimeCommandOptions,
-        ffParseUptimeJsonObject,
-        ffPrintUptime,
-        ffGenerateUptimeJsonResult,
-        ffPrintUptimeHelpFormat,
-        ffGenerateUptimeJsonConfig
-    );
-    ffOptionInitModuleArg(&options->moduleArgs);
+    options->moduleInfo = ffModuleInfo;
+    ffOptionInitModuleArg(&options->moduleArgs, "");
 }
 
 void ffDestroyUptimeOptions(FFUptimeOptions* options)
